#pragma once

#include <list>

using namespace std;

#define MAX_TEXT_LENGTH 1024

enum {
	TRANSCOMMAND_NOP = 0,
	TRANSCOMMAND_PTRCHEAT = 1,
	TRANSCOMMAND_OVERWRITE = 2,
	TRANSCOMMAND_SOW = 3,
	TRANSCOMMAND_SMSTR = 4
};

class COptionNode;
class CTransScriptParser;

//////////////////////////////////////////////////////////////////////////
// CTransCommand : CTransCommand ⺻ Ŭ
class CTransCommand
{
public:
	friend class CHookPoint;

protected:

	int		m_nTransMethod;
	BOOL	m_bUnicode;
	BOOL	m_bClipKor;
	BOOL	m_bClipJpn;
	BOOL	m_bRemoveSpace;
	BOOL	m_bTwoByte;
	BOOL	m_bSafe;

	BOOL	m_bAdjustLen;
	CString m_strLenPos;
	int		m_nLenPos;

	CString m_strArgScript;

public:
	void	*m_pArgText;
	void	*m_pPrevArgText;
	BYTE	m_pOrigTextBuf[MAX_TEXT_LENGTH*2];
	BYTE	m_byReserved[0x100];					// LEN()  
	BYTE	m_pTransTextBuf[MAX_TEXT_LENGTH*2];

public:
	static CTransCommand *CreateTransCommand(CString &strArgScript, COptionNode *pNode);	// CTransCommand  static Լ

protected:
	CTransCommand() : 
		m_nTransMethod(TRANSCOMMAND_NOP), m_bUnicode(FALSE), m_bClipKor(FALSE), 
		m_bClipJpn(FALSE), m_bRemoveSpace(FALSE), m_bTwoByte(FALSE), m_bSafe(FALSE), 
		m_bAdjustLen(FALSE), m_nLenPos(0),
		m_pArgText(NULL), m_pPrevArgText(NULL)
	{
		m_strArgScript.Empty();
		ZeroMemory(m_pOrigTextBuf, MAX_TEXT_LENGTH*2);
		ZeroMemory(m_byReserved, 0x100);
		ZeroMemory(m_pTransTextBuf, MAX_TEXT_LENGTH*2);
	}
	~CTransCommand() { RestoreBackup(); }

protected:
	// Set functions //
	void SetArgScript(LPCTSTR strScript){ m_strArgScript = strScript; };

public:
	//void SetDistFromEBP(int nDistFromESP){ m_nDistFromESP = nDistFromESP; };
	//void SetTransMethod(int nMethod){ m_nTransMethod = nMethod; };
/*	void SetUnicode(BOOL bValue){ m_bUnicode = bValue; };
	void SetClipKor(BOOL bClipKor){ m_bClipKor = bClipKor; };
	void SetClipJpn(BOOL bClipJpn){ m_bClipJpn = bClipJpn; };
	void SetRemoveSpace(BOOL bRemove){ m_bRemoveSpace = bRemove; };
	void SetTwoByte(BOOL bEnable){ m_bTwoByte = bEnable; };
	void SetSafe(BOOL bEnable){ m_bSafe = bEnable; };
*/
	//  ɼ մϴ.
	virtual void SetTransOption(CString strTransOption, COptionNode *pNode);
	// Ư ɼ (OVERWRITE(IGNORE) )  մϴ.
	virtual void SetSpecialOption(CString strSpecialOption, COptionNode *pNode) {}

	// Get functions //

	//int	 GetDistFromEBP(){ return m_nDistFromESP; };
	CString	GetArgScript(){ return m_strArgScript; };
	int  GetTransMethod(){ return m_nTransMethod; };
	BOOL GetUnicode(){ return m_bUnicode; };
	BOOL GetClipKor(){ return m_bClipKor; };
	BOOL GetClipJpn(){ return m_bClipJpn; };
	BOOL GetRemoveSpace(){ return m_bRemoveSpace; };
	BOOL GetTwoByte(){ return m_bTwoByte; };
	BOOL GetSafe(){ return m_bSafe; };

	// Execute functions //

	// (  ʿ )   մϴ.
	virtual void RestoreBackup() {}

	//  ڿ ġ ɴϴ. ߸ ڿ  NULL.
	virtual void *GetTextPoint(CTransScriptParser *pParser);

	// ޾ƿ ڿ   ִ ڿ Ȯմϴ.
	virtual BOOL IsValidTextPoint(void *pArgText, long nSize = 1024*1024*1024);

	// Ȥ  ڿ ٽ Ϸ Ȯմϴ.
	virtual BOOL IsDuplicateText(void *pArgText);

	// ڿ ̸  ڿ ̷ մϴ.
	virtual BOOL ChangeLength(void *pOrigBasePtr, CTransScriptParser *pParser, void *pNewBasePtr=NULL);

	//  ڿ  α׷ մϴ.
	virtual BOOL ApplyTranslatedText(void *pArgText, PREGISTER_ENTRY pRegisters, CTransScriptParser *pParser, BOOL bIsDuplicated) { return TRUE; }

};

//////////////////////////////////////////////////////////////////////////
// CTransCommandNOP : NOP CTransCommand Ȯ Ŭ
class CTransCommandNOP : public CTransCommand {
public:
	friend class CTransCommand;

protected:
	CTransCommandNOP() : CTransCommand()
		{ m_nTransMethod = TRANSCOMMAND_NOP; }
	~CTransCommandNOP() {}

public:

};

//////////////////////////////////////////////////////////////////////////
// CTransCommandPTRCHEAT : PTRCHEAT   CTransCommand Ȯ Ŭ
class CTransCommandPTRCHEAT : public CTransCommand {
public:
	friend class CTransCommand;

protected:
	CTransCommandPTRCHEAT() : CTransCommand()
		{ m_nTransMethod = TRANSCOMMAND_PTRCHEAT; }
	~CTransCommandPTRCHEAT() {}
public:
	virtual BOOL ChangeLength(void *pOrigBasePtr, CTransScriptParser *pParser, void *pNewBasePtr);

	virtual BOOL ApplyTranslatedText(void *pArgText, PREGISTER_ENTRY pRegisters, CTransScriptParser *pParser, BOOL bIsDuplicated);

};

//////////////////////////////////////////////////////////////////////////
// CTransCommandOVERWRITE : OVERWRITE   CTransCommand Ȯ Ŭ
class CTransCommandOVERWRITE : public CTransCommand {
public:
	friend class CTransCommand;

protected:
	BOOL	m_bAllSameText;
	BOOL	m_bIgnoreBufLen;

protected:
	CTransCommandOVERWRITE() : CTransCommand(), m_bAllSameText(FALSE), m_bIgnoreBufLen(FALSE)
		{ m_nTransMethod = TRANSCOMMAND_OVERWRITE; }
	~CTransCommandOVERWRITE() {}

	//   α׷ ϴ.
	BOOL OverwriteTextBytes( LPVOID pDest, LPVOID pSrc );

	//      ã  ϴ.
	BOOL SearchTextA(UINT_PTR ptrBegin, LPCSTR cszText, list<LPVOID>* pTextList);
	BOOL SearchTextW(UINT_PTR ptrBegin, LPCWSTR cwszText, list<LPVOID>* pTextList);

public:
	BOOL GetAllSameText(){ return m_bAllSameText; };
	BOOL GetIgnoreBufLen(){ return m_bIgnoreBufLen; };

	virtual void SetTransOption(CString strTransOption, COptionNode *pNode);
	virtual void SetSpecialOption(CString strSpecialOption, COptionNode *pNode);

	virtual BOOL ApplyTranslatedText(void *pArgText, PREGISTER_ENTRY pRegisters, CTransScriptParser *pParser, BOOL bIsDuplicated);

};


//////////////////////////////////////////////////////////////////////////
// CTransCommandSOW : SOW   CTransCommand Ȯ Ŭ
class CTransCommandSOW : public CTransCommand {
public:
	friend class CTransCommand;

protected:
	BYTE *m_pBackupPoint;
	BYTE *m_pBackupBuffer;
	UINT m_nBackupSize;

	// LEN() 
	int *m_pnBackupLenPoint;
	int m_nLenBytes;
	int m_nBackupLenPoint;
	bool m_bIsNullIncluded;

protected:
	CTransCommandSOW() : CTransCommand(), m_pBackupPoint(NULL), m_pBackupBuffer(NULL), m_nBackupSize(0),
						m_pnBackupLenPoint(NULL), m_nLenBytes(0), m_nBackupLenPoint(0), m_bIsNullIncluded(false)
		{ m_nTransMethod = TRANSCOMMAND_SOW; }
	~CTransCommandSOW() { RestoreBackup(); }

	//  Ÿ ϰ  ϴ.
	void DoBackupAndOverwrite(const PBYTE pBackupPoint, UINT nBackupSize);

public:
	virtual void RestoreBackup();
	virtual BOOL ChangeLength(void *pOrigBasePtr, CTransScriptParser *pParser, void *pNewBasePtr);
	virtual BOOL ApplyTranslatedText(void *pArgText, PREGISTER_ENTRY pRegisters, CTransScriptParser *pParser, BOOL bIsDuplicated);

};

//////////////////////////////////////////////////////////////////////////
// CTransCommandSMSTR : SMSTR   CTransCommand Ȯ Ŭ
class CTransCommandSMSTR : public CTransCommand {
public:
	friend class CTransCommand;

protected:
	BOOL m_bIgnoreBufLen;
	BOOL m_bMatchSize;

protected:
	CTransCommandSMSTR() : CTransCommand(), m_bIgnoreBufLen(FALSE), m_bMatchSize(FALSE)
		{ m_nTransMethod = TRANSCOMMAND_SMSTR; }
	~CTransCommandSMSTR() {}

	//  ̿  Ʈ ڿ ŵϴ.
	BOOL ApplySmartString( LPVOID pDest, LPVOID pSrc );

public:
	virtual void SetSpecialOption(CString strSpecialOption, COptionNode *pNode);

	virtual void *GetTextPoint(CTransScriptParser *pParser);

	virtual BOOL ChangeLength(void *pOrigBasePtr, CTransScriptParser *pParser, void *pNewBasePtr);

	virtual BOOL ApplyTranslatedText(void *pArgText, PREGISTER_ENTRY pRegisters, CTransScriptParser *pParser, BOOL bIsDuplicated);

};